Package: Synth
Version: 1.1-10
Date: 2026-04-28
Title: Synthetic Control Group Method for Comparative Case Studies
Authors@R: c(person(given = 'Jens', family = 'Hainmueller', role = c('aut','cre'), email = 'jhain@stanford.edu'), person(given = 'Alexis', family = 'Diamond', role = 'aut'))
Description: Implements the synthetic control group method for comparative case studies
 as described in Abadie and Gardeazabal (2003) and Abadie, Diamond, and Hainmueller
 (2010, 2011, 2014). The synthetic control method allows for effect estimation in
 settings where a single unit (a state, country, firm, etc.) is exposed to an event
 or intervention. It provides a data-driven procedure to construct synthetic control
 units based on a weighted combination of comparison units that approximates the
 characteristics of the unit that is exposed to the intervention. A combination of
 comparison units often provides a better comparison for the unit exposed to the
 intervention than any comparison unit alone.
Imports: kernlab, optimx, rgenoud
Suggests: testthat (>= 3.0.0)
Config/testthat/edition: 3
License: GPL (>= 2)
URL: https://web.stanford.edu/~jhain/
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2026-04-28 20:11:25 UTC; jhainmueller
Author: Jens Hainmueller [aut, cre],
  Alexis Diamond [aut]
Maintainer: Jens Hainmueller <jhain@stanford.edu>
Repository: CRAN
Date/Publication: 2026-04-29 06:40:25 UTC
Built: R 4.5.2; ; 2026-04-29 10:44:42 UTC; unix

import(kernlab)
import(optimx)
import(rgenoud)
importFrom("graphics", "abline", "legend", "lines")
importFrom("stats", "sd", "var")
export(synth, dataprep, gaps.plot, path.plot, synth.tab, fn.V, spec.pred.func, collect.optimx)

gaps.plot <-
function(
                      synth.res = NA,
                      dataprep.res = NA,
                      Ylab = c("Title"),
                      Xlab = c("Time"),
                      Main = c("Gaps: Treated - Synthetic"),
                      tr.intake = NA,
                      Ylim = NA,
                      Z.plot = FALSE
                      
                     )

              { 
                    if(Z.plot==FALSE)
                      { 
                     
                     if(sum(is.na(dataprep.res$Y1plot)) > 0)
                       {
                         stop(
                              "\n\n#####################################################",
                              "\nYou have missing Y data for the treated!\n\n"
                             )  
                        }
                     
                     if(sum(is.na(dataprep.res$Y0plot)) > 0)
                       {
                         stop(
                              "\n\n#####################################################",
                              "\nYou have missing Y data for the controls!\n\n"
                             )  
                        } 
                    
                       
                     # Compute Y-Gap: Treated - Synthtic              
                     gap <- dataprep.res$Y1plot - (dataprep.res$Y0plot %*% synth.res$solution.w)
                     
                     # Get Ylim right 
                     if(sum(is.na(Ylim))>0)
                      {
                        Ylim <- c(
                                  -(.3*max(abs(gap)) + max(abs(gap))),
                                  (.3*max(abs(gap))  + max(abs(gap)))
                                  )
                      }
                                                        
                     plot(
                          dataprep.res$tag$time.plot,
                          gap,
                          t="l",
                          col="black",
                          lwd=2,
                          main=Main,
                          ylab=Ylab,
                          xlab=Xlab,
                          ylim=Ylim,
                          xaxs="i",
                          yaxs="i"
                          )
         
                     } else {
                     
                     # Compute Y-Gap: Treated - Synthtic              
                     gap <- dataprep.res$Z1 - (dataprep.res$Z0 %*% synth.res$solution.w)
                     
                     # Get Ylim right 
                     if(sum(is.na(Ylim))>0)
                      {
                        Ylim <- c(
                                  -(.3*max(abs(gap)) + max(abs(gap))),
                                  (.3*max(abs(gap))  + max(abs(gap)))
                                  )
                      }
                                                        
                     plot(
                          dataprep.res$tag$time.optimize.ssr,
                          gap,
                          t="l",
                          col="black",
                          lwd=2,
                          main=Main,
                          ylab=Ylab,
                          xlab=Xlab,
                          ylim=Ylim,
                          xaxs="i",
                          yaxs="i"
                          )           
                  
                   }           
                     
                     abline(h=0, col="black",lty="dashed",lwd=2)
                     abline(v=tr.intake,col="black",lty="dotted",lwd=2)
           
            }

.onAttach <-
function (libname, pkgname)
{
   # echo output to screen
   packageStartupMessage("##\n## Synth Package: Implements Synthetic Control Methods.\n")
   packageStartupMessage("## See https://web.stanford.edu/~jhain/synthpage.html for additional information.\n\n")
}

synth.tab <-
function(
           synth.res    = NA,
           dataprep.res = NA,
           round.digit  = 3
           )

  {
    
    if(sum(is.na(synth.res))>0)
     {
      stop(
            "\n\n#####################################################",
            "\n No synth.res provided\n\n)"
            )
     }
     
         if(sum(is.na(dataprep.res))>0)
     {
      stop(
            "\n\n#####################################################",
            "\n No dataprep.res provided\n\n)"
            )
     }
     
     
    tab.v           <- matrix(round(synth.res$solution.v,round.digit),
                              nrow=length(synth.res$solution.v))
    rownames(tab.v) <- rownames(dataprep.res$X1)
    colnames(tab.v) <- c("v.weights")
   
    treat.no <-  dataprep.res$tag$treatment.identifier
    nmat     <-  dataprep.res$names.and.numbers[-which(
                                                       dataprep.res$names.and.numbers[,2]==treat.no)
                                               ,]
 
 
    tab.w           <- data.frame(
                                  round(synth.res$solution.w,round.digit),
                                  nmat
                                  )
    colnames(tab.w) <- c(
                         "w.weights",
                         "unit.names",
                         "unit.numbers"
                         )
  
    tab.loss           <- cbind(
                                synth.res$loss.w,
                                synth.res$loss.v
                                )
    rownames(tab.loss) <- NULL
    colnames(tab.loss) <- c(
                            "Loss W",
                            "Loss V"
                            )
    
    tab.pred <-     round(
                          cbind(
                                dataprep.res$X1,
                                as.matrix(dataprep.res$X0)%*%as.matrix(synth.res$solution.w),
                                apply(as.matrix(dataprep.res$X0),1,mean)
                                )
                          ,round.digit)
                          
    colnames(tab.pred) <- c(
                            "Treated",
                            "Synthetic",
                            "Sample Mean"
                            )
                        
    output <- list(
                   tab.pred = tab.pred,
                   tab.v = tab.v,
                   tab.w = tab.w,
                   tab.loss = tab.loss
                   )

    return(invisible(output))



  }

test_that("synth.tab() returns expected components", {
  d <- make_dataprep()
  fit <- synth(d, verbose = FALSE)
  tab <- synth.tab(synth.res = fit, dataprep.res = d)

  expect_named(tab, c("tab.pred", "tab.v", "tab.w", "tab.loss"))

  # tab.v: rownames = predictor names
  expect_equal(nrow(tab$tab.v), nrow(d$X1))
  expect_equal(colnames(tab$tab.v), "v.weights")

  # tab.w: data frame with weights, names, numbers
  expect_s3_class(tab$tab.w, "data.frame")
  expect_equal(colnames(tab$tab.w),
               c("w.weights", "unit.names", "unit.numbers"))
  expect_equal(nrow(tab$tab.w), ncol(d$X0))

  # tab.pred: predictor x (Treated, Synthetic, Sample Mean)
  expect_equal(colnames(tab$tab.pred),
               c("Treated", "Synthetic", "Sample Mean"))
  expect_equal(nrow(tab$tab.pred), nrow(d$X1))

  # Synthetic predictor values approximately match treated for the
  # well-fit example
  diff <- abs(tab$tab.pred[, "Treated"] - tab$tab.pred[, "Synthetic"])
  expect_true(all(diff < 5))   # generous, just sanity-checking
})

fn.V <-
function(
           variables.v = stop("variables.v missing"),
           X0.scaled = stop("X0.scaled missing"),
           X1.scaled = stop("X1.scaled missing"),
           Z0 = stop("Z0 missing"),
           Z1 = stop("Z1 missing"),
           margin.ipop = 0.0005,
           sigf.ipop = 5,
           bound.ipop = 10,
           quadopt = "ipop"
           )

  {
  
    # check quadopt
    if (!(quadopt %in% c("ipop", "LowRankQP"))) {
      stop("option quadopt must be \"ipop\" (\"LowRankQP\" is no longer supported)")
    }
  
    # rescale par
    #V <- diag( abs(variables.v)/sum(abs(variables.v)) )
    V <- diag(x=as.numeric(abs(variables.v)/sum(abs(variables.v))),
              nrow=length(variables.v),ncol=length(variables.v))
    
    # set up QP problem
    H <- t(X0.scaled) %*% V %*% (X0.scaled)
    a <- X1.scaled
    c <- -1*c(t(a) %*% V %*% (X0.scaled) )
    A <- t(rep(1, length(c)))
    b <- 1
    l <- rep(0, length(c))
    u <- rep(1, length(c))
    r <- 0
    
    # run QP and obtain w weights
    if (quadopt == "ipop") {
      res <- ipop(c = c, H = H, A = A, b = b, l = l, u = u, r = r,
                  bound = bound.ipop, margin = margin.ipop,
                  maxiter = 1000, sigf = sigf.ipop)
      solution.w <- as.matrix(primal(res))
    } else if (quadopt == "LowRankQP") {
      stop("LowRankQP is no longer supported; please use quadopt = \"ipop\" instead")
    }
            
    # compute losses
    loss.w <- as.numeric(t(X1.scaled - X0.scaled %*% solution.w) %*%
      (V) %*% (X1.scaled - X0.scaled %*% solution.w))

    loss.v <- as.numeric(t(Z1 - Z0 %*% solution.w) %*%
      ( Z1 - Z0 %*% solution.w ))
    loss.v <- loss.v/nrow(Z0)

    return(invisible(loss.v))
  }

test_that("synth() rejects deprecated quadopt = 'LowRankQP' (regression for 1.1-9 crash)", {
  d <- make_dataprep()
  expect_error(
    synth(d, quadopt = "LowRankQP", verbose = FALSE),
    "no longer supported"
  )
})

test_that("synth() rejects unknown quadopt", {
  d <- make_dataprep()
  expect_error(
    synth(d, quadopt = "newOptimizer", verbose = FALSE)
  )
})

test_that("fn.V() rejects deprecated quadopt = 'LowRankQP'", {
  d <- make_dataprep()
  big.df <- cbind(d$X0, d$X1)
  divisor <- sqrt(apply(big.df, 1, var))
  scaled <- t(t(big.df) %*% (1 / divisor *
                             diag(rep(nrow(big.df), 1))))
  X0s <- scaled[, 1:ncol(d$X0), drop = FALSE]
  X1s <- scaled[, ncol(scaled)]

  vstart <- rep(1 / nrow(d$X1), nrow(d$X1))
  expect_error(
    fn.V(variables.v = vstart, X0.scaled = X0s, X1.scaled = X1s,
         Z0 = d$Z0, Z1 = d$Z1, quadopt = "LowRankQP"),
    "no longer supported"
  )
})

dataprep <-
function(
                        foo = NULL,
                        predictors = NULL,
                        predictors.op = "mean",
                        special.predictors = NULL,
                        dependent = NULL,
                        unit.variable = NULL,
                        time.variable = NULL,
                        treatment.identifier = NULL,
                        controls.identifier = NULL,
                        time.predictors.prior = NULL,
                        time.optimize.ssr = NULL,
                        time.plot = time.optimize.ssr,
                        unit.names.variable = NA
                        )
  
  {

    if(is.data.frame(foo) == FALSE ){stop("\n No data.frame supplied in foo.\n")}

    # identify unit.variable
    if(mode(unit.variable) == "character"){unit.variable<-which(names(foo)==unit.variable)}
    if(is.null(unit.variable) == TRUE ||  mode(foo[,unit.variable])!="numeric")
      { stop("\n unit.variable not found as numeric variable in foo.\n")}
    if(length(unit.variable)!=1){stop("\n Please specify only one unit.variable\n")}

    # identify time.variable
    if(mode(time.variable) == "character"){time.variable<-which(names(foo)==time.variable)}
    if(is.null(time.variable) == TRUE || mode(foo[,time.variable])!="numeric" )
      {
       stop("\n time.variable not found as numeric variable in foo.\n")
      }
     if(length(time.variable)!=1){stop("\n Please specify only one time.variable\n")}
    # identify units
     # check if unit.name var is there (required if any identifier is given as character)
     if(
        mode(treatment.identifier) == "character" ||
        mode(controls.identifier)  == "character" ||
        is.na(unit.names.variable) == FALSE
        )
      {
       if(mode(unit.names.variable) == "character")
         {unit.names.variable<-which(names(foo)==unit.names.variable)}
       if(is.na(unit.names.variable) == TRUE || mode(foo[,unit.names.variable])!="character")
         {stop("\n unit.names.variable not found as character variable in foo.\n")}
      }
     # check treated
     if(length(treatment.identifier) != 1){stop("\n please specify a single treated unit\n")}
     if(mode(treatment.identifier) == "character")
      {
       if(treatment.identifier %in% foo[,unit.names.variable] == FALSE  )
        {stop("\n treated unit not found in unit.names.variable.\n")}
       tr.id = unique(foo[foo[,unit.names.variable] == treatment.identifier,unit.variable])
       if(length(tr.id)!=1)
         {stop(paste("\n treatment unit name",treatment.identifier,"refers to multiple numbers in unit.variable"))}
       treatment.identifier.name = treatment.identifier
       treatment.identifier = tr.id
      } else {
       if(treatment.identifier %in% foo[,unit.variable] == FALSE )
        {stop("\n treated unit not found in unit.variable.\n")}
        if(is.na(unit.names.variable) == TRUE )
         {
           treatment.identifier.name = treatment.identifier
          } else {
           treatment.identifier.name = unique(foo[treatment.identifier==foo[,unit.variable],unit.names.variable])
           if(length(treatment.identifier.name)>1)
            {stop("\n treatment.identifier refers to multiple names in unit.names.variable")}
          }
      }

     # check controls
     if(length(controls.identifier) < 2 ){stop("\n please specify at least two control units\n")}
     if(sum(duplicated(controls.identifier))>0)
      {stop("\n duplicate control units in controls.identifier\n")}
     if(mode(controls.identifier) == "character")
       { co.store <- c()
         for(i in controls.identifier)
          {
            if(i %in% foo[,unit.names.variable] == FALSE )
             {stop(paste("\n control unit",i,"not found in unit.names.variable"))}
            co.id = unique(foo[foo[,unit.names.variable] == i,unit.variable])
            if(length(co.id) != 1)
             {stop(paste("\n control unit name",i," refers to multiple numbers in unit.variable"))}
            co.store <- c(co.store,co.id)
          }
         controls.identifier.name = controls.identifier
         controls.identifier = co.store
        } else {
         co.store <- c()
         for(i in controls.identifier)
          {
            if(i %in% foo[,unit.variable] == FALSE )
             {stop(paste("\n control unit",i,"not found in unit.variable"))}
            if(is.na(unit.names.variable) == FALSE )
            {
             co.id = unique(foo[foo[,unit.variable] == i,unit.names.variable])
             if(length(co.id) != 1)
              {stop(paste("\n control unit number",i," refers to multiple names in unit.names.variable"))}
             co.store <- c(co.store,co.id)
             }
          }
         if(is.na(unit.names.variable) == FALSE )
          {
           controls.identifier.name = co.store
         } else {
           controls.identifier.name = controls.identifier
         }
        }

        # more checks
        if(treatment.identifier.name %in% controls.identifier.name)
         {stop("\n treated unit among controls\n")}
         
        if(sum(duplicated(c(controls.identifier.name,treatment.identifier.name))) > 0)
         {stop("\n duplicate unit.variable.names across units\n")}
         
        # sort first by unit, then by time variable
         foo[,time.variable] <- as.numeric(as.character(foo[,time.variable]))
         foo[,unit.variable] <- as.numeric(as.character(foo[,unit.variable]))
         foo <- foo[order(foo[,unit.variable], foo[,time.variable]),]

      # Get rows
       treatment.rows <- which(foo[,unit.variable] %in% treatment.identifier)
       control.rows   <- which(foo[,unit.variable] %in% controls.identifier)

      # check if panel is unbalanced  (possible cases where this too restrictive, but imposes discipline)
       balcheck <-       table(    foo[c(control.rows,treatment.rows),unit.variable],
                          foo[c(control.rows,treatment.rows),time.variable])         
        if( length(unique(balcheck)) != 1 || unique(balcheck)!= 1)
      {
        stop("\n Your panel, as described by unit.variable and time.variable, is unbalanced. Balance it and run again.\n")
      }

     # Now check and get time identifiers
      if(sum(is.null(time.predictors.prior))> 0)
       {stop("time.predictors.prior missing")}
      if(sum(is.null(time.optimize.ssr))>0)
       {stop("time.optimize.ssr missing")}
      if(sum(is.null(time.plot))>0)
       {stop("time.plot missing")}
       
      t.list <- list(time.predictors.prior,time.optimize.ssr,time.plot)
      names(t.list) <- c("predictors.prior","optimize.ssr","plot")
      for(i in 1:3)
       {
        if(mode(t.list[[i]]) != "numeric")
         {stop(paste("\n time",names(t.list[i]),"must be numeric\n"))}
        if(sum(duplicated(t.list[[i]]))>0)
         {stop(paste("\n duplicates in time",names(t.list[i]),"\n"))}
        if(length(t.list[[i]]) < 1)
         {stop(paste("\n specify at least one period in time",names(t.list[i]),"\n"))}
        for(p in t.list[[i]])
         {
          if(p %in% unique(foo[,time.variable]) == FALSE)
           stop(paste("\n time period ",p," from time.",names(t.list[i])," not found in time.variable\n",sep=""))
         }
        }
     
     # get time rows
     time.predictors.prior.rows <-
      which(foo[,time.variable] %in% time.predictors.prior)

     time.optimize.ssr.rows <-
      which(foo[,time.variable] %in% time.optimize.ssr)

    # Build X1 & X0 Predictor matrices
    
   # first, run check on regular predictors, if specified
   if(is.null(predictors)==FALSE)
   {
     # predictor checks
     if(sum(duplicated(predictors))>0)
      {stop("\n duplicates found in predictors \n")}

     for(i in predictors)
      {
        if(mode(foo[,i]) != "numeric")
         {stop(paste("\n predictor",i,"not found as numeric variable in foo \n"))}
      }

      if(mode(predictors)=="character")
       {
         pred.no <- c()
         for(i in 1:length(predictors)){pred.no <- c(pred.no,which(names(foo) == predictors[i]))}
         predictors <- pred.no
       }
    # X1 matrix for treated
    X1 <-
      data.frame(foo[
                        intersect(
                                  treatment.rows,
                                  time.predictors.prior.rows
                                  ),
                        predictors
                        ]
                    )
    colnames(X1) <- names(foo)[predictors]
    rownames(X1) <- time.predictors.prior

    # deep missing checker:
    for(i in 1:ncol(X1))
     {
       if(sum(is.na(X1[,i])) == nrow(X1))
        {stop(paste("\n predictor",names(X1)[i],"has missing data for all periods in time.predictors.prior\n"))}
        
      for(j in 1:nrow(X1))
       {
         if(is.na(X1[j,i])){
         cat(paste("\n Missing data- treated unit; predictor:",names(X1)[i],"; for period:",rownames(X1)[j],
         "\n","We ignore (na.rm = TRUE) all missing values for predictors.op.\n"))}
       }
     }

    # aggregate
    X1 <- apply(X1, 2, paste(predictors.op), na.rm = TRUE)
    X1 <- as.matrix(X1)
    rownames(X1) <- names(foo)[predictors]
    colnames(X1) <- treatment.identifier
    
    } else { # if no regular predictors are specified, pass a void matrix to special predictors
    
    X1 <- matrix(NA,0,1)
    colnames(X1) <- treatment.identifier 
    }
    
    # X0 matrix for controls
    if(is.null(predictors)==FALSE)
    {
    X0 <- data.frame(foo[intersect(control.rows,
                                      time.predictors.prior.rows
                                      ),
                            c(predictors, unit.variable)
                            ]
                        )
     checknames <- rep(time.predictors.prior,length(controls.identifier))

     # deep missing checker:
    for(i in 1:(ncol(X0)-1))
     {

       for(p in controls.identifier)
        {
         vals <- X0[X0[, ncol(X0)] == p, i]
         if (sum(is.na(vals)) == length(vals))
         {stop(paste("\n control unit:",p,"; predictor:",names(X0)[i],"has missing data for all periods in time.predictors.prior\n"))}
        }
       for(j in 1:nrow(X0))
       {
         if(is.na(X0[j,i])){
         cat(paste("\n Missing data - control unit:",X0[j,ncol(X0)],"; predictor:",names(X0)[i],"; for period:",checknames[j],
         "\n","We ignore (na.rm = TRUE) all missing values for predictors.op.\n"))}
       }
     }

    X0 <- split(X0, X0[,dim(X0)[2]])
    X0 <- sapply(X0, apply, 2, mean, na.rm = TRUE, simplify = TRUE)
    X0 <- as.matrix(X0[-dim(X0)[1],])
    
    
   # Take transpose in presence of a single predictor only
   if(dim(X0)[2]==1)
   {
    X0 <- t(X0)
    rownames(X0) <- names(foo)[predictors]
    colnames(X0) <- controls.identifier
   }

    } else { # if no regular predictors are specified, pass a void matrix to special predictors
    X0 <- matrix(NA,0,length(controls.identifier))
    #rownames(X0) <- names(foo)[predictors]
    colnames(X0) <- controls.identifier
    }

    # Add Special Predictors to X1 and X0

    if(is.null(special.predictors) == FALSE)
      {
        if(is.list(special.predictors) == FALSE)
         {stop("\nspecial.predictors is not a list object\n")}

        for(i in 1:length(special.predictors))
          {
          
           # checks for special predictors
            if(is.list(special.predictors[[i]]) == FALSE)
             {stop(paste("\n special.predictor number",i,"is not a list object\n"))}

            if(length(special.predictors[[i]]) != 3)
             {stop(paste("\n special.predictor number",i,"is misspecified (requires predictor name, time periods, and name of operation\n"))}

           # name check
            sp.name <- special.predictors[[i]][[1]]
            if(is.na(sp.name) || length(sp.name) != 1)
            {stop(paste("\n predictor name",sp.name,"of special.predictor number",i,"misspecified\n"))}
            
           # availability check
           if(mode(foo[,sp.name]) != "numeric")
            {stop(paste("\n special predictor named ",sp.name,"not found as numeric variable in foo \n"))}
           
           # time check
            sp.time <- special.predictors[[i]][[2]]
            if(mode(sp.time) != "numeric")
            {stop(paste("\n for special predictor:",sp.name," (which is special.predictor number",i,") the time period is misspecified\n"))}
            if(sum(duplicated(sp.time))>0)
             {stop(paste("\n for special predictor:",sp.name," (which is special.predictor number",i,") time period contains duplicates\n"))}
            if(length(sp.time)<1)
             {stop(paste("\n for special predictor:",sp.name," (which is special.predictor number",i,") specify at least one time period\n"))}
             
           # time availability check
           for(p in sp.time)
            {
             if(p %in% unique(foo[,time.variable]) == FALSE)
              {stop(paste("\n for special predictor:",sp.name," (which is special.predictor number",i,") time period",p,"not found in time.variable\n"))}
            }

           # operator check
           sp.op <- special.predictors[[i]][[3]]
            if(mode(sp.op) != "character" || length(sp.op) !=1 )
            {stop(paste("\n for special predictor:",sp.name," (which is special.predictor number",i,") the operator is misspecified\n"))}

           # now go and built matrices
           spf <- spec.pred.func(list.object = special.predictors[[i]],
                                  tr.numb = treatment.identifier,
                                  co.numb = controls.identifier,
                                  unit.var = unit.variable,
                                  time.var = time.variable,
                                  foo.object = foo,
                                  X0.inner = X0,
                                  X1.inner = X1
                                  )

            X0 <- spf[[1]]
            X1 <- spf[[2]]

          }

      }

   # no predictors check
    if(nrow(X0)==0)
     {stop("No predictors specified. Please specify at least one predictor")}

   # Dependent Variable Pretreatment

    if(is.null(dependent))
     {stop("\n dependent variable is missing")}

    if(mode(foo[,dependent]) != "numeric")
         {stop(paste("\n dependent variable",dependent,"not found as numeric variable in foo \n"))}

    if(mode(dependent) == "character")
      {dependent <- which(is.element(names(foo), dependent))}
    #foo[,dependent] <- as.numeric(as.character(foo[,dependent]))

    Z1 <- foo[
              intersect(
                        treatment.rows,
                        time.optimize.ssr.rows
                        ),
              dependent
              ]

    Z1 <- as.matrix(Z1)

    # missing check
    if(sum(is.na(Z1)) == length(Z1))
     {stop("\n treated unit: dependent variable is missing for all periods in time.optimize.ssr.rows \n")}

    rownames(Z1) <- time.optimize.ssr
    colnames(Z1) <- treatment.identifier
    
    for(i in 1:length(Z1))
      {
        if(is.na(Z1[i]))
        {stop(paste("\n treated unit: dependent variable is missing for period",rownames(Z1)[i],"in time.optimize.ssr.rows \n"))}
      }

    Z0 <-
      as.matrix(foo[
                    intersect(
                              control.rows,
                              time.optimize.ssr.rows
                              ),
                    c(dependent)
                    ]
                )

    Z0 <- matrix(
                 Z0,
                 byrow = FALSE,
                 nrow = length(time.optimize.ssr),
                 ncol = length(controls.identifier),
                 )
                 
    colnames(Z0)<- controls.identifier
    rownames(Z0)<- time.optimize.ssr

    for(i in 1:ncol(Z0))
      {
        for(j in 1:nrow(Z0))
        {
        if(is.na(Z0[j,i]))
        {stop(paste("\n control unit:",colnames(Z0)[i],"; dependent variable",dependent,"is missing for period",rownames(Z0)[j],"in time.optimize.ssr.rows \n"))}
        }
      }
    

     # dependent Variable Plot
        time.plot.rows <-
          which(is.element(foo[,time.variable], time.plot ) == TRUE)

        Y1plot <- foo[intersect(treatment.rows, time.plot.rows), dependent]
        Y1plot <- as.matrix(Y1plot)
        rownames(Y1plot) <- time.plot
        colnames(Y1plot) <- treatment.identifier

     if(sum(is.na(Y1plot)) == length(Y1plot))
     {stop("\n treated unit: dependent variable is missing for all periods in time.plot \n")}

      for(i in 1:length(Y1plot))
       {
        if(is.na(Y1plot[i]))
        {stop(paste("\n treated unit: dependent variable is missing for period",rownames(Y1plot)[i],"in time.plot \n"))}
       }

        Y0plot <- as.matrix(foo[
                                intersect(control.rows, time.plot.rows),
                                c(dependent)
                                ]
                            )

        Y0plot  <- matrix(
                          Y0plot,
                          byrow = FALSE,
                          nrow = length(time.plot),
                          ncol = length(controls.identifier),
                          )


        rownames(Y0plot) <- time.plot
        colnames(Y0plot) <- controls.identifier
     

        # table with unit names
              names.and.numbers <-
        data.frame(c(treatment.identifier.name,controls.identifier.name),
                   c(treatment.identifier,controls.identifier))



      names(names.and.numbers) <- c(
                                    "unit.names",
                                    "unit.numbers"
                                    )



  #######################
       
  tag <- list(
              foo = as.character(foo),
              predictors = predictors,
              predictors.op = predictors.op,
              special.predictors = special.predictors,
              dependent = dependent,
              unit.variable = unit.variable,
              time.variable = time.variable,
              treatment.identifier = treatment.identifier,
              controls.identifier = controls.identifier,
              time.predictors.prior = time.predictors.prior,
              time.optimize.ssr = time.optimize.ssr,
              time.plot = time.plot,
              unit.names.variable = unit.names.variable
              )

######################################

  output <- list(
                 X0 = X0,
                 X1 = X1,
                 Z0 = Z0,
                 Z1 = Z1,
                 Y0plot = Y0plot,
                 Y1plot = Y1plot,
                 names.and.numbers = names.and.numbers,
                 tag = tag
                 )

  return(invisible(output))

}

test_that("dataprep() builds the expected matrix shapes", {
  d <- make_dataprep()

  expect_type(d, "list")
  expect_named(d, c("X0", "X1", "Z0", "Z1",
                    "Y0plot", "Y1plot",
                    "names.and.numbers", "tag"))

  # X1: predictors x 1 treated unit. We have 3 regular + 3 special = 6 rows.
  expect_equal(dim(d$X1), c(6L, 1L))
  expect_equal(colnames(d$X1), "7")

  # X0: predictors x n_controls
  expect_equal(dim(d$X0), c(6L, 6L))
  expect_setequal(as.integer(colnames(d$X0)),
                  c(29L, 2L, 13L, 17L, 32L, 38L))

  # Z1: time.optimize.ssr x 1 treated
  expect_equal(dim(d$Z1), c(7L, 1L))

  # Z0: time.optimize.ssr x n_controls
  expect_equal(dim(d$Z0), c(7L, 6L))

  # No NAs anywhere in the inputs
  expect_false(anyNA(d$X1))
  expect_false(anyNA(d$X0))
  expect_false(anyNA(d$Z1))
  expect_false(anyNA(d$Z0))
})

test_that("dataprep() rejects malformed inputs", {
  expect_error(
    dataprep(foo = "not a data frame"),
    "data.frame"
  )

  expect_error(
    dataprep(foo = synth.data,
             predictors = c("X1", "X2"),
             dependent = "Y",
             unit.variable = "unit.num",
             time.variable = "year",
             treatment.identifier = 7,
             controls.identifier = c(29),  # only one control
             time.predictors.prior = c(1984:1989),
             time.optimize.ssr = c(1984:1990),
             time.plot = 1984:1996),
    "at least two control"
  )
})

test_that("dataprep() handles single-period special predictors", {
  d <- dataprep(
    foo = synth.data,
    predictors = c("X1"),
    predictors.op = "mean",
    dependent = "Y",
    unit.variable = "unit.num",
    time.variable = "year",
    special.predictors = list(
      list("Y", 1985, "mean")
    ),
    treatment.identifier = 7,
    controls.identifier = c(29, 2, 13, 17),
    time.predictors.prior = c(1984:1989),
    time.optimize.ssr = c(1984:1990),
    time.plot = 1984:1996
  )
  expect_equal(nrow(d$X1), 2L)  # 1 regular + 1 special
})

synth <-
function(           data.prep.obj = NULL,
                      X1 = NULL,
                      X0 = NULL,
                      Z0 = NULL,
                      Z1 = NULL,
                      custom.v = NULL,
                      optimxmethod = c("Nelder-Mead","BFGS"),
                      genoud = FALSE,
                      quadopt = "ipop",
                      Margin.ipop = 0.0005,
                      Sigf.ipop = 5,
                      Bound.ipop = 10,
                      verbose = FALSE,
                       ...
                      )
  { 
    # Retrieve dataprep objects
    if (!is.null(data.prep.obj)) {
      if (verbose) cat("\nX1, X0, Z1, Z0 all come directly from dataprep object.\n\n")
      X1 <- data.prep.obj$X1
      Z1 <- data.prep.obj$Z1
      X0 <- data.prep.obj$X0
      Z0 <- data.prep.obj$Z0
    } else {
      if (verbose) cat("X1, X0, Z1, Z0 were individually input (not dataprep object).\n\n")
    }

     # routine checks
     store <- list(X1=X1,X0=X0,Z1=Z1,Z0=Z0)
     for(i in 1:4){
     if(is.null(store[[i]]))
      {stop(paste("\n",names(store)[i],"is missing \n"))}
     if(sum(is.na(store[[i]]))>0)
      {stop(paste("\n NAs in",names(store)[i],"\n"))}
     if(is.matrix(store[[i]]) == FALSE)
      {stop(paste("\n",names(store)[i],"is not a matrix object\n"))}
     }

    # geometry checks
    if(ncol(X1)!=1){stop("\n Please specify only one treated unit: X1 has to have ncol= 1")}
    if(ncol(Z1)!=1){stop("\n Please specify only one treated unit: Z1 has to have ncol= 1")}

    if(ncol(X0)<2){stop("\n Please specify at least two control units: X0 has to have ncol >= 2 ")}
    if(ncol(Z0)<2){stop("\n Please specify at least two control units: Z0 has to have ncol >= 2")}

    if(nrow(Z0)!=nrow(Z1)){stop("\n Different number of periods for treated and controls: nrow(Z0) unequal nrow(Z1)")}
    if(nrow(X0)!=nrow(X1)){stop("\n Different number of predictors for treated and controls: nrow(X0) unequal nrow(X1)")}

    if(nrow(X0)==0){stop("No predictors specified. Please specify at least one predictor")}
    if(nrow(Z0)==0){stop("No periods specified for Z1 and Z0. Please specify at least one period")}

    if(0 %in% apply(X0,1,sd))
     {stop("\n At least one predictor in X0 has no variation across control units. Please remove this predictor.")}

    # Normalize X
    nvarsV <- dim(X0)[1]
    big.dataframe <- cbind(X0, X1)
    divisor <- sqrt(apply(big.dataframe, 1, var))
    scaled.matrix <-
      t(t(big.dataframe) %*% ( 1/(divisor) *
                              diag(rep(dim(big.dataframe)[1], 1)) ))

    X0.scaled <- scaled.matrix[, 1:ncol(X0), drop = FALSE]
    X1.scaled <- scaled.matrix[, ncol(scaled.matrix)]


    # check if custom v weights are supplied or
    # if only on predictor is specified,
    # we jump to quadratic optimization over W weights
    # if not start optimization over V and W
    if(is.null(custom.v) & nrow(X0) != 1)
      {
      
      # two attempts for best V are made:
      # equal weights and regression based starting values
      if (verbose) cat("\n****************",
                       "\n searching for synthetic control unit  \n", "\n")

      if (genoud == TRUE) { # if user wants genoud as well
      # we run genoud first
      if (verbose) cat("\n****************",
                       "\n genoud() requested for optimization\n", "\n")

      rgV.genoud <- rgenoud::genoud(
                             fn.V, 
                             nvarsV, 
                             X0.scaled = X0.scaled,
                             X1.scaled = X1.scaled,
                             Z0 = Z0,
                             Z1 = Z1,
                             quadopt = quadopt,
                             margin.ipop = Margin.ipop,
                             sigf.ipop = Sigf.ipop,
                             bound.ipop = Bound.ipop
                             )
      SV1 <- rgV.genoud$par  # and use these as starting values

      if (verbose) cat("\n****************",
                       "\n genoud() finished, now running local optimization using optim()\n", "\n")

      } else {
      # if we don't use genoud first: set of starting values: equal weights
      SV1 <- rep(1/nvarsV,nvarsV)
      }
      
      # now we run optimization
      all.methods <- FALSE
      if(sum(optimxmethod %in% c("All"))==1){ all.methods <- TRUE }
     rgV.optim.1 <- optimx(par=SV1, fn=fn.V,
                             gr=NULL, hess=NULL, 
                             method=optimxmethod, itnmax=NULL, hessian=FALSE,
                             control=list(kkt=FALSE,
                                          starttests=FALSE,
                                          dowarn=FALSE,
                                          all.methods=all.methods),
                             X0.scaled = X0.scaled,
                             X1.scaled = X1.scaled,
                             Z0 = Z0,
                             Z1 = Z1,
                             quadopt = quadopt,
                             margin.ipop = Margin.ipop,
                             sigf.ipop = Sigf.ipop,
                             bound.ipop = Bound.ipop
                            )
      # get minimum
      if(verbose==TRUE){print(rgV.optim.1)}
      rgV.optim.1 <- collect.optimx(rgV.optim.1,"min")
      
      # second set of starting values: regression method 
      # will sometimes not work because of collinear Xs
      # so it's wrapped in a try command
      Xall <- cbind(X1.scaled,X0.scaled)
      Xall <- cbind(rep(1,ncol(Xall)),t(Xall))
      Zall <- cbind(Z1,Z0)
      Beta <- try(solve(t(Xall)%*%Xall)%*%t(Xall)%*%t(Zall),silent=TRUE)
      
      # if inverses did not work, we
      # stick with first results    
      if(inherits(Beta,"try-error")) 
       {
        rgV.optim <- rgV.optim.1
       } else {
      # otherwise we run a second optimization with regression starting values
        Beta <- Beta[-1,]
        V    <- Beta%*%t(Beta)
        SV2  <- diag(V)
        SV2 <- SV2 / sum(SV2)
  
      rgV.optim.2 <- optimx(par=SV2, fn=fn.V,
                             gr=NULL, hess=NULL, 
                             method=optimxmethod, itnmax=NULL, hessian=FALSE,
                             control=list(kkt=FALSE,
                                          starttests=FALSE,
                                          dowarn=FALSE,
                                          all.methods=all.methods),
                             X0.scaled = X0.scaled,
                             X1.scaled = X1.scaled,
                             Z0 = Z0,
                             Z1 = Z1,
                             quadopt = quadopt,
                             margin.ipop = Margin.ipop,
                             sigf.ipop = Sigf.ipop,
                             bound.ipop = Bound.ipop
                            )
      # get minimum
      if(verbose==TRUE){print(rgV.optim.2)}
      rgV.optim.2 <- collect.optimx(rgV.optim.2,"min")
  
      # ouput
      if(verbose == TRUE){
      cat("\n Equal weight loss is:",rgV.optim.1$value,"\n")
      cat("\n Regression Loss is:",rgV.optim.2$value,"\n")
      }       
      # and keep the better optim results    
      if(rgV.optim.1$value < rgV.optim.2$value) 
       {
        rgV.optim <- rgV.optim.1
       } else {
        rgV.optim <- rgV.optim.2
       }
      } # close if statement for second regression based optimization attempt
     
      # final V weights from optimization
      solution.v   <- abs(rgV.optim$par)/sum(abs(rgV.optim$par))
     } else { # jump here if only optimize over W

     if (verbose) cat("\n****************",
                      "\n optimization over w weights: computing synthetic control unit \n", "\n\n")

     if (nrow(X0) == 1) {
       custom.v <- 1 # only one predictor: V is the identity matrix
     } else {
       # user-supplied v
       if (verbose) cat("\n****************",
                        "\n v weights supplied manually: computing synthetic control unit \n", "\n\n")
       if (length(custom.v) != nvarsV) {
         stop("custom.v misspecified: length(custom.v) != nrow(X0)")
       }
       if (mode(custom.v) != "numeric") {
         stop("custom.v must be numeric")
       }
     }

    # enter solution.V
    rgV.optim  <- NULL
    solution.v <- abs(custom.v)/sum(custom.v)
    
  } # close else statment for by-passing V optimization


    # last step: now recover solution.w
    V <- diag(x=as.numeric(solution.v),nrow=nvarsV,ncol=nvarsV)
    H <- t(X0.scaled) %*% V %*% (X0.scaled)
    a <- X1.scaled
    c <- -1*c(t(a) %*% V %*% (X0.scaled) )
    A <- t(rep(1, length(c)))
    b <- 1
    l <- rep(0, length(c))
    u <- rep(1, length(c))
    r <- 0

    if (quadopt == "ipop") {
      res <- ipop(c = c, H = H, A = A, b = b, l = l, u = u, r = r,
                  margin = Margin.ipop, maxiter = 1000,
                  sigf = Sigf.ipop, bound = Bound.ipop)
      solution.w <- as.matrix(primal(res))
    } else if (quadopt == "LowRankQP") {
      stop("LowRankQP is no longer supported; please use quadopt = \"ipop\" instead")
    } else {
      stop(sprintf("Unknown quadopt = \"%s\"; supported value is \"ipop\"", quadopt))
    }

    rownames(solution.w) <- colnames(X0)
    colnames(solution.w) <- "w.weight"
    names(solution.v) <- rownames(X0)

    loss.w <- t(X1.scaled - X0.scaled %*% solution.w) %*%
      V %*% (X1.scaled - X0.scaled %*% solution.w)

    loss.v <-
      t(Z1 - Z0 %*% as.matrix(solution.w)) %*%
        (Z1 - Z0 %*% as.matrix(solution.w)) 
    loss.v <- loss.v/nrow(Z0)      
 
    # produce viewable output
    if (verbose) {
      cat("\n****************",
          "\n****************",
          "\n****************",
          "\n\nMSPE (LOSS V):", loss.v,
          "\n\nsolution.v:\n", round(as.numeric(solution.v), 10),
          "\n\nsolution.w:\n", round(as.numeric(solution.w), 10),
          "\n\n")
    }
        
    optimize.out <- list(
                         solution.v = solution.v,
                         solution.w = solution.w,
                         loss.v = loss.v,
                         loss.w = loss.w,
                         custom.v = custom.v,
                         rgV.optim = rgV.optim
                         )

    return(invisible(optimize.out))

  }

test_that("synth() with dataprep object converges and weights sum to 1", {
  d <- make_dataprep()
  fit <- synth(d, verbose = FALSE)

  expect_type(fit, "list")
  expect_named(fit,
               c("solution.v", "solution.w", "loss.v", "loss.w",
                 "custom.v", "rgV.optim"))
  expect_equal(sum(fit$solution.w), 1, tolerance = 1e-6)
  expect_true(all(fit$solution.w >= 0 - 1e-6))
  expect_true(all(fit$solution.w <= 1 + 1e-6))

  # V weights should also sum to 1 after normalization in synth().
  expect_equal(sum(fit$solution.v), 1, tolerance = 1e-6)
  expect_true(all(fit$solution.v >= 0))
})

test_that("synth() matrix interface gives the same answer as dataprep interface", {
  d <- make_dataprep()
  fit_obj <- synth(d, verbose = FALSE)
  fit_mat <- synth(X1 = d$X1, X0 = d$X0,
                   Z1 = d$Z1, Z0 = d$Z0,
                   verbose = FALSE)

  expect_equal(fit_obj$solution.w, fit_mat$solution.w, tolerance = 1e-6)
  expect_equal(fit_obj$solution.v, fit_mat$solution.v, tolerance = 1e-6)
  expect_equal(fit_obj$loss.v, fit_mat$loss.v, tolerance = 1e-6)
})

test_that("synth() with custom.v skips V optimization", {
  d <- make_dataprep()
  cv <- rep(1, nrow(d$X1)) / nrow(d$X1)
  fit <- synth(d, custom.v = cv, verbose = FALSE)
  expect_equal(as.numeric(fit$solution.v), cv, tolerance = 1e-12)
  expect_null(fit$rgV.optim)
})

test_that("synth() rejects malformed inputs", {
  d <- make_dataprep()

  expect_error(
    synth(X1 = d$X1, X0 = d$X0, Z0 = d$Z0, Z1 = NULL, verbose = FALSE),
    "Z1 is missing"
  )

  # X0 with no variation across controls
  X0bad <- d$X0
  X0bad[1, ] <- 1
  expect_error(
    synth(X1 = d$X1, X0 = X0bad, Z0 = d$Z0, Z1 = d$Z1, verbose = FALSE),
    "no variation"
  )
})

test_that("synth(verbose = FALSE) produces no console output", {
  d <- make_dataprep()
  out <- capture.output(synth(d, verbose = FALSE))
  expect_length(out, 0L)
})

test_that("synth(verbose = TRUE) prints the MSPE summary block", {
  d <- make_dataprep()
  out <- capture.output(synth(d, verbose = TRUE))
  expect_true(any(grepl("MSPE", out)))
  expect_true(any(grepl("solution.v", out)))
  expect_true(any(grepl("solution.w", out)))
})

test_that("synth() matrix interface is silent at verbose = FALSE", {
  d <- make_dataprep()
  out <- capture.output(
    synth(X1 = d$X1, X0 = d$X0, Z1 = d$Z1, Z0 = d$Z0, verbose = FALSE)
  )
  expect_length(out, 0L)
})

## Shared fixtures across the test suite. Loaded automatically by testthat.

data(synth.data, package = "Synth", envir = environment())

# A canonical dataprep object — matches s1_toy_panel from
# dev/01_capture_baseline.R so changes here are caught by the regression
# script as well.
make_dataprep <- function() {
  dataprep(
    foo = synth.data,
    predictors = c("X1", "X2", "X3"),
    predictors.op = "mean",
    dependent = "Y",
    unit.variable = "unit.num",
    time.variable = "year",
    special.predictors = list(
      list("Y", 1991, "mean"),
      list("Y", 1985, "mean"),
      list("Y", 1980, "mean")
    ),
    treatment.identifier = 7,
    controls.identifier = c(29, 2, 13, 17, 32, 38),
    time.predictors.prior = c(1984:1989),
    time.optimize.ssr = c(1984:1990),
    unit.names.variable = "name",
    time.plot = 1984:1996
  )
}

path.plot <-
function(
                      synth.res = NA,
                      dataprep.res = NA,
                      tr.intake = NA,
                      Ylab = c("Y Axis"),
                      Xlab = c("Time"),
                      Ylim = NA ,
                      Legend=c("Treated","Synthetic"),
                      Legend.position=c("topright"),
                      Main = NA,
                      Z.plot = FALSE
                      )

              { 
                     
                     if(Z.plot==FALSE)
                      { 
                     
                     if(sum(is.na(dataprep.res$Y1plot)) > 0)
                       {
                         stop(
                              "\n\n#####################################################",
                              "\nYou have missing Y data for the treated!\n\n"
                             )  
                        }
                     
                     if(sum(is.na(dataprep.res$Y0plot)) > 0)
                       {
                         stop(
                              "\n\n#####################################################",
                              "\nYou have missing Y data for the controls!\n\n"
                             )  
                        } 
                     
                     
                     y0plot1 <- dataprep.res$Y0plot %*% synth.res$solution.w

                     # Get Ylim right
                     if (sum(is.na(Ylim)) > 0) {
                        Y.max <- max(c(y0plot1, dataprep.res$Y1plot))
                        Y.min <- min(c(y0plot1, dataprep.res$Y1plot))
                        # Pad each end by 30% of the data range so negative
                        # Y.min extends downward (not toward zero).
                        Y.pad <- 0.3 * (Y.max - Y.min)
                        if (Y.pad == 0) Y.pad <- 0.3 * max(abs(c(Y.min, Y.max)), 1)
                        Ylim <- c(Y.min - Y.pad, Y.max + Y.pad)
                     }
                              
                      
                     plot(
                      dataprep.res$tag$time.plot
                     ,dataprep.res$Y1plot,
                     t="l",
                     col="black",
                     lwd=2,
                     main=Main,
                     ylab=Ylab,
                     xlab=Xlab,xaxs="i",yaxs="i",ylim=Ylim)
                     
                     lines(
                           dataprep.res$tag$time.plot,
                           y0plot1 ,
                           col="black",
                           lty="dashed",
                           lwd=2,
                           cex=4/5
                           )
                                  
                 } else {
                 
                 z0plot <- dataprep.res$Z0 %*% synth.res$solution.w

                     # Get Ylim right
                     if (sum(is.na(Ylim)) > 0) {
                        Y.max <- max(c(z0plot, dataprep.res$Z1))
                        Y.min <- min(c(z0plot, dataprep.res$Z1))
                        Y.pad <- 0.3 * (Y.max - Y.min)
                        if (Y.pad == 0) Y.pad <- 0.3 * max(abs(c(Y.min, Y.max)), 1)
                        Ylim <- c(Y.min - Y.pad, Y.max + Y.pad)
                     }
                              
                      
                     plot(
                     dataprep.res$tag$time.optimize.ssr
                     ,z0plot,
                     t="l",
                     col="black",
                     lwd=2,
                     main=Main,
                     ylab=Ylab,
                     xlab=Xlab,xaxs="i",yaxs="i",ylim=Ylim)
                     
                     lines(
                           dataprep.res$tag$time.optimize.ssr,
                           dataprep.res$Z1,
                           col="black",
                           lty="dashed",
                           lwd=2,
                           cex=4/5
                           ) 

            }

            abline(v=tr.intake,lty=3,col="black",lwd=2)
            
            if(sum(is.na(Legend))==0)
                     {
                     legend(Legend.position,legend=Legend,
                     lty=1:2,col=c("black","black"),lwd=c(2,2),cex=6/7)
                     }
            
     }

spec.pred.func <-
function(
                           list.object = NULL,
                           tr.numb = NULL,
                           co.numb = NULL,
                           unit.var = NULL,
                           time.var = NULL,
                           foo.object = NULL,
                           X0.inner = NULL,
                           X1.inner = NULL
                           )
  {

    special.units.tr <-
      which(is.element(foo.object[,unit.var], tr.numb) == TRUE)

    special.units.co <-
      which(is.element(foo.object[,unit.var], co.numb) == TRUE)

    special.times <-
      which(is.element(foo.object[,time.var], list.object[[2]]) == TRUE)

    if(mode(list.object[[1]]) == "character")
      {
        list.object[[1]] <- which(names(foo.object) == list.object[[1]])
      }

    # create predictor name:
    name.predictor <- names(foo.object)[list.object[[1]]]
    if(length(list.object[[2]])>1)
      {
      name.predictor <-
      paste("special.", name.predictor, ".",
            paste(list.object[[2]][1],".",
                  list.object[[2]][length(list.object[[2]])]
                                  , sep = ""), sep = "")

      } else {

      name.predictor <-
      paste("special.", name.predictor, ".",
            paste(list.object[[2]]
            , sep = ""), sep = "")
      }

      X1.special <- as.matrix(foo.object[
                                       intersect(
                                                 special.units.tr,
                                                 special.times
                                                ),
                                       list.object[[1]]
                                       ]
                            )

    rownames(X1.special) <-
      foo.object[
                 intersect(special.units.tr, special.times),
                 time.var
                 ]
    colnames(X1.special) <- tr.numb

   # deep missing checker:
    for(i in 1:ncol(X1.special))
     {
       if(sum(is.na(X1.special[,i])) == nrow(X1.special))
        {stop(paste("\n special predictor",name.predictor,"has missing data for all periods in time.predictors.prior\n"))}

      for(j in 1:nrow(X1.special))
       {
         if(is.na(X1.special[j,i])){
         cat(paste("\n Missing data: treated unit; special predictor:",name.predictor,"; for period:",rownames(X1.special)[j],
         "\n","We ignore (na.rm = TRUE) all missing values for predictors.op.\n"))}
       }
     }

    # aggregate if only one time period is there
    if(length(list.object[[2]]) > 1)
      {
       X1.special <- apply(X1.special,
                            2,
                            paste(list.object[[3]]),
                            na.rm = TRUE
                            )
      }
    X1.special <- t(as.matrix(X1.special))
    X1.inner <- rbind(X1.inner, X1.special)
    rownames(X1.inner)[nrow(X1.inner)] <- name.predictor

   # controls
    X0.special <-
      as.matrix(foo.object[intersect(
                                     special.units.co,
                                     special.times
                                     ),
                           c(list.object[[1]])
                           ]
                )

    X0.special <-
      matrix(X0.special[,1], byrow = FALSE, ncol = length(co.numb))

    # Define row and column names

    row.names(X0.special) <-
      foo.object[intersect(special.units.tr, special.times), time.var]

    colnames(X0.special) <- co.numb

    # deep missing checker:
    for(i in 1:ncol(X0.special))
     {
      if(sum(is.na(X0.special[,i])) == nrow(X0.special))
       {stop(paste("\n control unit:",co.numb[i],"; special predictor:",name.predictor,"has missing data for all periods specified:",list.object[[2]],"\n"))}

       for(j in 1:nrow(X0.special))
       {
         if(is.na(X0.special[j,i])){
         cat(paste("\n Missing data - control unit:",co.numb[i],"; special predictor:",name.predictor,"; for period:",rownames(X0.special)[j],
         "\n","We ignore (na.rm = TRUE) all missing values for predictors.op.\n"))}
       }
     }

    # Continue with object creation
    if(length(list.object[[2]]) > 1)
     {
        X0.special <- apply(
                            X0.special,
                            2,
                            paste(list.object[[3]]),
                            na.rm = TRUE
                            )
                            
        X0.special <- t(as.matrix(X0.special))
     }

    X0.inner <- rbind(X0.inner,X0.special)
    rownames(X0.inner)[nrow(X0.inner)] <- name.predictor




    # Prepare output
    special.output <- list(X0.inner = X0.inner,
                           X1.inner = X1.inner
                           )

    return(invisible(special.output))

  }

collect.optimx <-
function(res,opt="min"){ 

# reorder dataframe for best solution
if(opt=="min"){
  res <- res[order(res$value,decreasing=FALSE),]
}
if(opt=="max"){
  res <- res[order(res$value,decreasing=TRUE),]
}
 
# index for value
val <- which(colnames(res)=="value") 
# index for pars
pars <- 1:(val-1)

# extract best solution
out <- list(out.list=res,
            par=res[1,pars],
            value=res[1,val]
            )
return(invisible(out))

}

\name{path.plot}
\alias{path.plot}
\title{Plots Outcome Trajectories for Treated Unit and its Synthetic Control Unit}
\description{
  This function plots the trajectories of the outcome variable for the treated unit and the synthetic control group constructed by \code{\link{synth}} and \code{\link{dataprep}}. The user can specify whether the whole 
  time period or only the pretreatment period should be plotted. 
}
\usage{
path.plot(synth.res = NA,
dataprep.res = NA,
tr.intake = NA,
Ylab = c("Y Axis"),
Xlab = c("Time"),
Ylim = NA,
Legend=c("Treated","Synthetic"),
Legend.position=c("topright"),
Main = NA,
Z.plot = FALSE)
}
\arguments{
  \item{synth.res}{Output list created by \code{\link{synth}}.}
  \item{dataprep.res}{Output list created by \code{\link{dataprep}}.}
  \item{tr.intake}{Optional scalar to indicate the time of treatment intake with a vertical line.}
  \item{Ylab}{Optional label for Y axis.}
  \item{Xlab}{Optional label for X axis.}
  \item{Ylim}{Optional Ylim.}
  \item{Main}{Optional main title.}
  \item{Legend}{Optional legend text (e.g. c("Treated","Synthetic")); see ?legend for details.}
  \item{Legend.position}{Optional legend position (e.g. "bottomright"); see ?legend for details.}
  \item{Z.plot}{Flag. If true, only pretreatment period is plotted.}
}
\details{
  The trajectory of the outcome for the synthetic control group is calculated as:
  dataprep.res$Y0plot\%*\% synth.res$solution.w. You can use this calculation
  to construct custom made plots.
}
\value{
  The plot of trajectories.
}
\references{

Abadie, A., Diamond, A., Hainmueller, J. (2014). Comparative Politics and the Synthetic Control Method. \emph{American Journal of Political Science} Forthcoming 2014.

Synthetic : An R Package for Synthetic Control Methods in Comparative Case Studies. \emph{Journal of Statistical Software} 42 (13) 1--17.
       
Abadie, A., Diamond, A., Hainmueller, J. (2011). Synth: An R Package for Synthetic Control Methods in Comparative Case Studies. \emph{Journal of Statistical Software} 42 (13) 1--17.

Abadie A, Diamond A, Hainmueller J (2010). Synthetic Control Methods for Comparative Case Studies: Estimating the Effect of California's Tobacco Control Program. \emph{Journal of the American Statistical Association} 105 (490) 493--505.
     
Abadie, A. and Gardeazabal, J. (2003) Economic Costs of Conflict: A Case Study of the Basque Country \emph{American Economic Review} 93 (1) 113--132.

            }
   
\author{Jens Hainmueller and Alexis Diamond}
\seealso{\code{\link{dataprep}}, \code{\link{gaps.plot}}, \code{\link{synth}}, \code{\link{synth.tab}}
}

\name{synth.tab}
\alias{synth.tab}
\title{Creates Tables that Summarize Results of Synthetic Control Group Method}
\description{
  This function is called after \code{\link{dataprep}} and \code{\link{synth}} in order to create tables summarizing the results of the 
  run of the synthetic control method. The result tables can be latexed directly.}
\usage{
synth.tab(synth.res    = NA,
          dataprep.res = NA,
          round.digit  = 3)
}

\arguments{
  \item{synth.res}{The list resulting from the call to \code{\link{synth}}.}
  \item{dataprep.res}{The list resulting from the call to \code{\link{dataprep}}.}
  \item{round.digit}{Integer for rounding in tables.}
}
\details{NA}
\value{
  \item{tab.v}{The matrix that contains the table of V-weights and respective variable names.}
  \item{tab.w}{The matrix that contains the table of W-weights and respective unit numbers and possibly names.}
  \item{tab.loss}{The matrix that contains the table of W-loss and V-loss}
}
\references{

Abadie, A., Diamond, A., Hainmueller, J. (2014). Comparative Politics and the Synthetic Control Method. \emph{American Journal of Political Science} Forthcoming 2014.

Synthetic : An R Package for Synthetic Control Methods in Comparative Case Studies. \emph{Journal of Statistical Software} 42 (13) 1--17.
       
Abadie, A., Diamond, A., Hainmueller, J. (2011). Synth: An R Package for Synthetic Control Methods in Comparative Case Studies. \emph{Journal of Statistical Software} 42 (13) 1--17.

Abadie A, Diamond A, Hainmueller J (2010). Synthetic Control Methods for Comparative Case Studies: Estimating the Effect of California's Tobacco Control Program. \emph{Journal of the American Statistical Association} 105 (490) 493--505.
     
Abadie, A. and Gardeazabal, J. (2003) Economic Costs of Conflict: A Case Study of the Basque Country \emph{American Economic Review} 93 (1) 113--132.
  
            }
   
\author{Jens Hainmueller and Alexis Dimaond}
\seealso{\code{\link{synth}}, \code{\link{dataprep}}, \code{\link{gaps.plot}}, \code{\link{path.plot}}
}

\name{synth.data}
 \docType{data}
 \alias{synth.data}
 \title{Panel Data to demonstrate the use of the Synthetic Control Method}
 \description{This artificial panel data set is used to demonstrate the use of the 
  Synthetic Control Method. 
          }
\usage{data(synth.data)}
\format{A dataframe made up of 8 units: 1 treated (no 7) and 7 control (no. 2,7,13,17,29,32,36,38)
 , 3 predictors (X1, X2, X3),  21 time periods (1980 - 2000), a unit.names.variable column ("names") and an outcome variable column (Y). All columns have column names.}
\keyword{datasets}

\name{fn.V}
\alias{fn.V}
\title{
Loss Function for nested optimization of W and V weights
}
\description{
Loss function for the nested optimization of W and V weights used for constructing synthetic control groups according to the methods outlined in Abadie and Gardeazabal (2003) and Abadie, Diamond, Hainmueller (2010, 2011, 2014) (see references). This function is called by \code{\link{synth}} internally, and should not be called manually by a normal user.
}
\usage{
fn.V(variables.v = stop("variables.v missing"),
X0.scaled = stop("X0.scaled missing"), 
X1.scaled = stop("X1.scaled missing"), 
Z0 = stop("Z0 missing"), 
Z1 = stop("Z1 missing"), 
margin.ipop = 5e-04, 
sigf.ipop = 5, 
bound.ipop = 10, 
quadopt = "ipop")
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{variables.v}{
1 by k a vector of v weights.
}
  \item{X0.scaled}{
matrix of controls' predictor data. nrows = number of predictors and (possibly) special predictors. ncols = number of control units.
}
  \item{X1.scaled}{matrix of treated predictor data. nrows = number of predictors and (possibly) special predictors. ncols = one.
}
  \item{Z0}{matrix of controls' outcome data for the pre-treatment periods over which MSPE is to be minimized. nrows = number of pre-treatment periods. ncols = number of control units.
}
  \item{Z1}{
matrix of treated outcome data for the pre-treatment periods over which MSPE is to be minimized. nrows = number of pre-treatment periods. ncols = one.
}
  \item{margin.ipop}{
setting for ipop optimization routine: how close we get to the constrains 
(see \code{\link[kernlab:ipop]{ipop}} for details)
}
  \item{sigf.ipop}{
setting for ipop optimization routine: Precision (default: 7 significant figures) 
(see \code{\link[kernlab:ipop]{ipop}} for details)
}
  \item{bound.ipop}{
setting for ipop optimization routine: Clipping bound for the variables 
(see \code{\link[kernlab:ipop]{ipop}} for details)
}
  \item{quadopt}{
string vector that specifies the routine for quadratic optimization over w weights. 
possible values are "ipop" (see \code{\link[kernlab:ipop]{ipop}} for details)
}

}
\value{
A scalar that contains the function value.
}
\references{

Abadie, A., Diamond, A., Hainmueller, J. (2014). Comparative Politics and the Synthetic Control Method. \emph{American Journal of Political Science} Forthcoming 2014.

Synthetic : An R Package for Synthetic Control Methods in Comparative Case Studies. \emph{Journal of Statistical Software} 42 (13) 1--17.
       
Abadie, A., Diamond, A., Hainmueller, J. (2011). Synth: An R Package for Synthetic Control Methods in Comparative Case Studies. \emph{Journal of Statistical Software} 42 (13) 1--17.

Abadie A, Diamond A, Hainmueller J (2010). Synthetic Control Methods for Comparative Case Studies: Estimating the Effect of California's Tobacco Control Program. \emph{Journal of the American Statistical Association} 105 (490) 493--505.
     
Abadie, A. and Gardeazabal, J. (2003) Economic Costs of Conflict: A Case Study of the Basque Country \emph{American Economic Review} 93 (1) 113--132.
  
}


\author{
Jens Hainmueller and Alexis Diamond
}

\seealso{ \code{\link{synth}}, \code{\link{dataprep}}, \code{\link{gaps.plot}}, \code{\link{path.plot}},
 \code{\link{synth.tab}}
}

\name{gaps.plot}
\alias{gaps.plot}
\title{Plots Gap in Outcome Trajectories between the Treated its Synthetic Control Unit}
\description{ This function plots the gaps in the trajectories of the outcome variable for the treated unit and the 
  synthetic control group constructed by \code{\link{synth}} and \code{\link{dataprep}}. The user can specify whether
  the whole time period or only the pre-treatment period should be plotted.
}
\usage{gaps.plot(synth.res = NA,
       dataprep.res = NA,
       Ylab = c("Title"),
       Xlab = c("Time"),
       Main = c("Gaps: Treated - Synthetic"),
       tr.intake = NA,
       Ylim = NA,
       Z.plot = FALSE)
}
\arguments{
  \item{synth.res}{Output list created by \code{\link{synth}}.}
  \item{dataprep.res}{Output list created by \code{\link{dataprep}}.}
  \item{tr.intake}{Optional scalar to indicate the time of treatment intake with a vertical line.}
  \item{Ylab}{Optional label for Y axis.}
  \item{Xlab}{Optional label for X axis.}
  \item{Ylim}{Optional Ylim.}
  \item{Main}{Optional main title.}
  \item{Z.plot}{Flag. If true, only pretreatment period is plotted.}
}
\details{ The trajectory of the outcome for the synthetic control group is calculated as:
  dataprep.res$Y0plot \%*\% synth.res$solution.w. You can use this calculation to construct custom made plots.
}
\value{The plot of trajectories.}
\references{

Abadie, A., Diamond, A., Hainmueller, J. (2014). Comparative Politics and the Synthetic Control Method. \emph{American Journal of Political Science} Forthcoming 2014.

Synthetic : An R Package for Synthetic Control Methods in Comparative Case Studies. \emph{Journal of Statistical Software} 42 (13) 1--17.
       
Abadie, A., Diamond, A., Hainmueller, J. (2011). Synth: An R Package for Synthetic Control Methods in Comparative Case Studies. \emph{Journal of Statistical Software} 42 (13) 1--17.

Abadie A, Diamond A, Hainmueller J (2010). Synthetic Control Methods for Comparative Case Studies: Estimating the Effect of California's Tobacco Control Program. \emph{Journal of the American Statistical Association} 105 (490) 493--505.
     
Abadie, A. and Gardeazabal, J. (2003) Economic Costs of Conflict: A Case Study of the Basque Country \emph{American Economic Review} 93 (1) 113--132.
  
            }
   
\author{Jens Hainmueller and Alexis Diamond}
\seealso{\code{\link{dataprep}}, \code{\link{synth}}, \code{\link{path.plot}}, \code{\link{synth.tab}}
}

\name{basque}
 \docType{data}
 \alias{basque}
 \title{Panel Data from Spanish Regions to demonstrate the use of the Synthetic Control Method}
 \description{The dataset contains information from 1955--1997
on 17 Spanish regions. It was used by Abadie and Gardeazabal (2003), which studied the economic effects of
conflict, using the terrorist conflict in the Basque Country as a case study. This
paper used a combination of other Spanish regions to construct a synthetic
control region resembling many relevant economic characteristics of
the Basque Country before the onset of political terrorism in the 1970s.
The data contains per-capita GDP (the outcome variable), as well as
population density, sectoral production, investment, and human capital
(the predictor variables) for the relevant years, and is used here to demonstrate the implementation
of the synthetic control method with the synth library.}
\usage{data(basque)}
\format{A panel dataframe made up of 18 units: 1 treated (no 17; the Basque country) and 16 control regions (no. 2-16,18).
Region no. 1 is the average for the whole country of Spain. 1 outcome variable (gdpcap). 13 predictor variables 
(6 sectoral production shares, 6 highest educational attainment categories, population density, and the investment rate).
Region names and numbers are stored in regionno and regionname. 42 time periods (1955 - 1997).
All columns have self-explanatory column names. For reference the variables are:
\itemize{
\item \preformatted{regionno}: Region Number.
\item \preformatted{regionname}: Region Name.
\item \preformatted{year}: Year.
\item \preformatted{gdpcap}: real GDP per capita (in 1986 USD, thousands).
\item \preformatted{sec.agriculture}: production in agriculture, forestry, and fishing sector as a percentage of
total production.
\item \preformatted{sec.energy}: production in energy and water sector as a percentage of
total production.
\item \preformatted{sec.industry}: production in industrial sector as a percentage of
total production.
\item \preformatted{sec.construction}: production in construction and engineering sector as a percentage of
total production.
\item \preformatted{sec.energy}: production in marketable services sector as a percentage of
total production.
\item \preformatted{sec.energy}: production in Nonmarketable services sector as a percentage of
total production.
\item \preformatted{school.illit}: number of illiterate persons.
\item \preformatted{school.prim}: number of persons with primary education or without studies.
\item \preformatted{school.med}: number of persons with some high school education.
\item \preformatted{school.high}: number of persons with high school degree.
\item \preformatted{school.post.high}: number of persons with tertiary education. 
\item \preformatted{popdens}: population density (persons per square kilometer).
\item \preformatted{invest}: gross total investment as a share of GDP.
}
}
\source{Abadie, A. and Gardeazabal, J. (2003) Economic Costs of Conflict: A Case Study of the Basque Country \emph{American Economic Review} 93 (1) 113--132.
            
Abadie, A., Diamond, A., Hainmueller, J. (2011). Synth: An R Package for Synthetic Control Methods in Comparative Case Studies. \emph{Journal of Statistical Software} 42 (13) 1--17.

}

\name{synth}
\alias{synth}
\title{
Constructs synthetic control units for comparative case studies
}
\description{
Implements the synthetic control method for causal inference in comparative case studies as developed in Abadie and Gardeazabal (2003) and Abadie, Diamond, and Hainmueller (2010, 2011, 2014). \code{\link{synth}} estimates the effect of an intervention by comparing the evolution of an aggregate outcome for a unit affected by the intervention to the evolution of the same aggregate outcome for a synthetic control group.

\code{\link{synth}} constructs this synthetic control group by searching for a weighted combination of control units chosen to approximate the unit affected by the intervention in terms of characteristics that are predictive of the outcome. The evolution of the outcome for the resulting synthetic control group is an estimate of the counterfactual of what would have been observed for the affected unit in the absence of the intervention.

\code{\link{synth}} can also be used to conduct a variety of placebo and permutation tests that produce informative inference regardless of the number of available comparison units and the number of available time periods. See Abadie and Gardeazabal (2003), Abadie, Diamond, and Hainmueller (2010, 2011, 2014) for details.

\code{\link{synth}} requires the user to supply four matrices as its main arguments. These matrices are named X0, X1, Z1, and Z0 accordingly. X1 and X0 contain the predictor values for the treated unit and the control units respectively. Z1 and Z0 contain the outcome variable for the pre-intervention period for the treated unit and the control units respectively. The pre-intervention period refers to the time period prior to the intervention, over which the mean squared prediction error (MSPE) should be minimized. The MSPE refers to the squared deviations between the outcome for the treated unit and the synthetic control unit summed over all pre-intervention periods specified in Z1 and Z0.

Creating the matrices X1, X0, Z1, and Z0 from a (panel) dataset can be tedious. Therefore, the \pkg{Synth} package offers a preparatory function called \code{\link{dataprep}} that allows the user to easily create all inputs required for \code{\link{synth}}. By first calling \code{\link{dataprep}}, the user creates a single list object called \code{data.prep.obj} that contains all essential data elements to run \code{synth}.

A usual sequence of commands to implement the synthetic control method is to first call \code{\link{dataprep}} to prepare the data, then call \code{\link{synth}} to construct the synthetic control group, and finally summarize results using the functions \code{\link{synth.tab}}, \code{\link{path.plot}}, or \code{\link{gaps.plot}}.

An example of this sequence is provided in the documentation to \code{\link{dataprep}}. This procedure is strongly recommended. Alternatively, the user may provide their own preprocessed data matrices and load them into \code{\link{synth}} via the X0, X1, Z1, and Z0 arguments. In this case, no \code{data.prep.obj} should be specified.

The output from \code{\link{synth}} is a list object that contains the weights on predictors (\code{solution.V}) and weights on control units (\code{solution.W}) that define contributions to the synthetic control unit.
}
\usage{
synth(data.prep.obj = NULL,
      X1 = NULL, X0 = NULL,
      Z0 = NULL, Z1 = NULL,
      custom.v = NULL,
      optimxmethod = c("Nelder-Mead", "BFGS"),
      genoud = FALSE, quadopt = "ipop",
      Margin.ipop = 5e-04,
      Sigf.ipop = 5,
      Bound.ipop = 10,
      verbose = FALSE, ...)
}
\arguments{
  \item{data.prep.obj}{
The object produced by \code{\link{dataprep}}. This object contains all information about X0, X1, Z1, and Z0. If \code{data.prep.obj} is supplied, none of X0, X1, Z1, and Z0 should be manually specified.
}
  \item{X1}{Matrix of treated predictor data. Rows correspond to predictors, columns to a single treated unit.}
  \item{X0}{Matrix of control units’ predictor data. Rows correspond to predictors, columns to control units (>=2).}
  \item{Z1}{Matrix of treated outcome data for the pre-treatment periods over which MSPE is minimized.}
  \item{Z0}{Matrix of control units’ outcome data for the pre-treatment periods over which MSPE is minimized.}
  \item{custom.v}{
Vector of weights for predictors supplied by the user. Uses \code{synth} to bypass optimization for \code{solution.V}. See Details.
}
  \item{optimxmethod}{
Character vector specifying optimization algorithms to be used. Permissible values are all optimization algorithms currently implemented in the function \code{\link[optimx:optimx]{optimx}} (see that function for details). If multiple algorithms are specified, \code{\link{synth}} will run all chosen algorithms and return the best-performing result. If \code{"All"} is specified, all algorithms in \code{\link[optimx:optimx]{optimx}} are tried.
}
  \item{genoud}{
Logical flag. If \code{TRUE}, \code{\link{synth}} performs a two-step optimization. In the first step, \code{\link[rgenoud:genoud]{genoud}} is used to obtain a global solution. In the second step, the results are passed to the algorithm(s) chosen in \code{optimxmethod} for local optimization. This two-step procedure is slower but may yield lower loss for irregular search spaces.
}
  \item{quadopt}{
Character specifying the routine for quadratic optimization over W weights. Possible value: \code{"ipop"} (see \code{\link[kernlab:ipop]{ipop}} for details).
}
  \item{Margin.ipop}{
Setting for the \code{ipop} optimization routine: how close we get to the constraints (see \code{\link[kernlab:ipop]{ipop}} for details).
}
  \item{Sigf.ipop}{
Setting for the \code{ipop} optimization routine: precision in significant figures (default: 7; see \code{\link[kernlab:ipop]{ipop}} for details).
}
  \item{Bound.ipop}{
Setting for the \code{ipop} optimization routine: clipping bound for variables (see \code{\link[kernlab:ipop]{ipop}} for details).
}
  \item{verbose}{Logical flag. If \code{TRUE}, intermediate results are displayed.}
  \item{\dots}{Additional arguments passed to \code{\link[optimx:optimx]{optimx}} or \code{\link[rgenoud:genoud]{genoud}}.}
}
\details{
As proposed in Abadie and Gardeazabal (2003) and Abadie, Diamond, and Hainmueller (2010), \code{\link{synth}} searches for the set of weights that generate the best-fitting convex combination of control units. The predictor weight matrix V is chosen among positive definite diagonal matrices such that MSPE is minimized for the pre-intervention period.

Alternatively, the user may supply a vector of V weights based on a subjective assessment of the predictive power of the variables in X1 and X0. In this case, specify \code{custom.V} in \code{\link{synth}}, and the optimization over V matrices is bypassed.
}
\value{
  \item{solution.v}{Vector of predictor weights.}
  \item{solution.w}{Vector of weights across control units.}
  \item{loss.v}{MSPE from optimization over V and W weights.}
  \item{loss.w}{Loss from optimization over W weights.}
  \item{custom.v}{If specified, returns the user-supplied weight vector.}
  \item{rgV.optim}{Results from \code{optimx()} minimization, useful for diagnostics.}
}
\references{
Abadie, A., Diamond, A., Hainmueller, J. (2014). Comparative Politics and the Synthetic Control Method. \emph{American Journal of Political Science}.

Abadie, A., Diamond, A., Hainmueller, J. (2011). Synth: An R Package for Synthetic Control Methods in Comparative Case Studies. \emph{Journal of Statistical Software}, 42(13), 1–17.

Abadie, A., Diamond, A., Hainmueller, J. (2010). Synthetic Control Methods for Comparative Case Studies: Estimating the Effect of California's Tobacco Control Program. \emph{Journal of the American Statistical Association}, 105(490), 493–505.

Abadie, A., and Gardeazabal, J. (2003). Economic Costs of Conflict: A Case Study of the Basque Country. \emph{American Economic Review}, 93(1), 113–132.
}
\author{Jens Hainmueller and Alexis Diamond}
\seealso{
\code{\link{dataprep}}, \code{\link{gaps.plot}}, \code{\link{path.plot}}, \code{\link{synth.tab}}
}
\examples{
\donttest{
data(synth.data)
dataprep.out <- dataprep(foo = synth.data,
   predictors = c("X1","X2","X3"),
   predictors.op = "mean",
   dependent = "Y",
   unit.variable = "unit.num",
   time.variable = "year",
   special.predictors = list(
      list("Y",1991,"mean"),
      list("Y",1985,"mean"),
      list("Y",1980,"mean")),
   treatment.identifier = 7,
   controls.identifier = c(29,2,13,17,32,38),
   time.predictors.prior = 1984:1989,
   time.optimize.ssr = 1984:1990,
   unit.names.variable = "name",
   time.plot = 1984:1996)
synth.out <- synth(dataprep.out)
path.plot(dataprep.res = dataprep.out, synth.res = synth.out)
gaps.plot(dataprep.res = dataprep.out, synth.res = synth.out)
}
}

\name{dataprep}
\alias{dataprep}
\title{Constructs a list of matrices from panel dataset to be loaded into synth()}
\description{
The \code{\link{synth}} function takes a standard panel dataset and produces a list of data objects necessary for running \code{\link{synth}} and other Synth package functions to construct synthetic control groups according to the methods outlined in Abadie and Gardeazabal (2003) and Abadie, Diamond, Hainmueller (2010, 2011, 2014) (see references and example). 

User supplies a dataframe ("foo"), chooses predictors, special predictors (explained below), the operators that act upon these predictors, the dependent variable, identifies the columns associated with unit numbers, time periods (and unit names, when available), as well as the treated unit, the control units, the time-period over which to select the predictors, the time-period over which to optimize, and the time-period over which outcome data should be plotted.

The output of \code{\link{dataprep}} contains a list of matrices. This list object can be directly loaded into \code{\link{synth}}. 
}
\usage{dataprep(foo = NULL, predictors = NULL,
          predictors.op = "mean", special.predictors = NULL,
          dependent = NULL, unit.variable = NULL,
          time.variable = NULL, treatment.identifier = NULL,
          controls.identifier = NULL, time.predictors.prior = NULL,
          time.optimize.ssr = NULL, time.plot = time.optimize.ssr,
          unit.names.variable = NA)
}

\arguments{
  \item{foo}{The dataframe with the panel data.}

  \item{predictors}{A vector of column numbers or column-name character
    strings that identifies the predictors' columns. All predictors have to be numeric.}
  \item{predictors.op}{A character string identifying the method (operator) to be used
    on the predictors.  Default is "mean". rm.na = T is hardwired into
    the code. See *Details*.}
  \item{special.predictors}{A list object identifying additional numeric predictors and their associated pre-treatment years and operators (analogous to
    ``predictors.op'' above).  See *Details*.}
  \item{dependent}{A scalar identifying the column number or column-name character string that corresponds to the numeric dependent (outcome) variable.}
  \item{unit.variable}{A scalar identifying the column number or column-name character string associated unit numbers. The unit.varibale has to be numeric.}
  \item{time.variable}{A scalar identifying column number or column-name character string associated with period (time) data. The time variable has to be numeric.} 
  \item{treatment.identifier}{A scalar identifying the ``unit.variable'' number or a character string
    giving the ``unit.name ''of the treated unit. If a character is supplied, a unit.names.variable also has to be supplied to identify the treated unit.}
  \item{controls.identifier }{ A scalar identifying the ``unit.variable'' numbers or a vector of character strings giving the ``unit.name''s of control units. If a character is supplied, a unit.names.variable also has to be supplied to identify the control units unit.}
  \item{time.predictors.prior }{A numeric vector identifying the pretreatment periods over which the values for the outcome predictors should be averaged.}
  \item{time.optimize.ssr}{A numeric vector identifying the periods of the dependent variable over which the loss function 
should be minimized (i.e. the periods over which mean squared prediction error (MSPE) , that is the sum of squared residuals between treated and the synthetic control unit, are minimized.}
  \item{time.plot}{A vector identifying the periods over which results are to be plotted
    with \code{\link{gaps.plot}} and \code{\link{path.plot}}.}
  \item{unit.names.variable}{A scalar or column-name character string identifying the column with the names of the units. This variable has to be of mode character. 
  }
}
\details{ The predictors.op argument is a character string that provides a
  function (eg., "mean", "median", etc.) identifying the name of the
  operator to be applied to the predictors over the given time period. 

  The special.predictors argument is a list object that contains one or
  more lists of length = 3.  The required components of each of these lists are:

  (a) scalar column number associated with that predictor
  (b) vector of time-period number(s) desired (eg., 1998:2003)
  (c) character-string identifying the name of the operation 
  to be applied (ie., "mean", "median", etc.)

  eg.,
  special.predictors <- list(listc(x1, 1990:2000, "mean"),
                             listc(x2, 1980:1983, "median"),
                             listc(x3, 1980, "mean")   
                             )
  
  indicates that predictor x1, should be used with its values averaged over periods 1990:2000; 
predicator x2 should be used with its median values over periods 1980:1983; x3 should be used with the values from period 1980 only.

}
\value{ 

  \item{X1}{matrix of treated predictor data.
    nrows = number of predictors and (possibly) special predictors. ncols = one.
  }

  \item{X0}{matrix of controls' predictor data. nrows = number of predictors and (possibly) special predictors. ncols = number of control units.
  }


  \item{Z1}{matrix of treated outcome data for the pre-treatment periods over which MSPE is to be minimized. nrows = number of pre-treatment periods. ncols = one.
  }

  \item{Z0}{matrix of controls' outcome data for the pre-treatment periods over which MSPE is to be minimized.
    nrows = number of pre-treatment periods.
    ncols = number of control units.
  }

  \item{Y1plot}{matrix of outcome data for treated unit to be used for results plotting. nrows = number of periods. ncols = one.
  }

  \item{Y0plot}{matrix of outcome data for control units to be used for results plotting. nrows = number of periods. ncols = number of control units. 
  }


  \item{names.and.numbers}{dataframe with two columns showing all unit numbers and
    corresponding unit names.
  }
  \item{tag}{a list of all arguments in initial function call.}
}
\references{

Abadie, A., Diamond, A., Hainmueller, J. (2014). Comparative Politics and the Synthetic Control Method. \emph{American Journal of Political Science} Forthcoming 2014.

Synthetic : An R Package for Synthetic Control Methods in Comparative Case Studies. \emph{Journal of Statistical Software} 42 (13) 1--17.
       
Abadie, A., Diamond, A., Hainmueller, J. (2011). Synth: An R Package for Synthetic Control Methods in Comparative Case Studies. \emph{Journal of Statistical Software} 42 (13) 1--17.

Abadie A, Diamond A, Hainmueller J (2010). Synthetic Control Methods for Comparative Case Studies: Estimating the Effect of California's Tobacco Control Program. \emph{Journal of the American Statistical Association} 105 (490) 493--505.
     
Abadie, A. and Gardeazabal, J. (2003) Economic Costs of Conflict: A Case Study of the Basque Country \emph{American Economic Review} 93 (1) 113--132.
  
}
   
\author{Jens Hainmueller and Alexis Diamond}
\seealso{ \code{\link{synth}}, \code{\link{gaps.plot}}, \code{\link{path.plot}}, \code{\link{synth.tab}} 
}
\examples{
\donttest{
## The usual sequence of commands is:
## 1. dataprep() for matrix-extraction
## 2. synth() for the construction of the synthetic control group
## 3. synth.tab(), gaps.plot(), and path.plot() to summarize the results
## Below we provide two examples.

## First Example: Toy panel dataset

# load data
data(synth.data)

# create matrices from panel data that provide inputs for synth()
dataprep.out<-
  dataprep(
   foo = synth.data,
   predictors = c("X1", "X2", "X3"),
   predictors.op = "mean",
   dependent = "Y",
   unit.variable = "unit.num",
   time.variable = "year",
   special.predictors = list(
      list("Y", 1991, "mean"),
      list("Y", 1985, "mean"),
      list("Y", 1980, "mean")
                            ),
   treatment.identifier = 7,
   controls.identifier = c(29, 2, 13, 17, 32, 38),
   time.predictors.prior = c(1984:1989),
   time.optimize.ssr = c(1984:1990),
   unit.names.variable = "name",
   time.plot = 1984:1996
   )

## run the synth command to identify the weights
## that create the best possible synthetic 
## control unit for the treated.
synth.out <- synth(dataprep.out)

## there are two ways to summarize the results
## we can either access the output from synth.out directly
round(synth.out$solution.w,2)
# contains the unit weights or
synth.out$solution.v 
## contains the predictor weights. 

## the output from synth opt 
## can be flexibly combined with 
## the output from dataprep to 
## compute other quantities of interest
## for example, the period by period 
## discrepancies between the 
## treated unit and its synthetic control unit
## can be computed by typing
gaps<- dataprep.out$Y1plot-(
        dataprep.out$Y0plot\%*\%synth.out$solution.w
        ) ; gaps

## also there are three convenience functions to summarize results.
## to get summary tables for all information 
## (V and W weights plus balance btw. 
## treated and synthetic control) use the 
## synth.tab() command
synth.tables <- synth.tab(
      dataprep.res = dataprep.out,
      synth.res = synth.out)
print(synth.tables)

## to get summary plots for outcome trajectories 
## of the treated and the synthetic control unit use the 
## path.plot() and the gaps.plot() commands

## plot in levels (treated and synthetic)
path.plot(dataprep.res = dataprep.out,synth.res = synth.out)

## plot the gaps (treated - synthetic)
gaps.plot(dataprep.res = dataprep.out,synth.res = synth.out)

}
\donttest{
## Second example: The economic impact of terrorism in the
## Basque country using data from Abadie and Gardeazabal (2003)
## see JSS paper in the references details

data(basque)

# dataprep: prepare data for synth
dataprep.out <-
  dataprep(
  foo = basque
  ,predictors= c("school.illit",
                 "school.prim",
                 "school.med",
                 "school.high",
                 "school.post.high"
                 ,"invest"
                 )
   ,predictors.op = c("mean")
   ,dependent     = c("gdpcap")
   ,unit.variable = c("regionno")
   ,time.variable = c("year")
   ,special.predictors = list(
    list("gdpcap",1960:1969,c("mean")),                            
    list("sec.agriculture",seq(1961,1969,2),c("mean")),
    list("sec.energy",seq(1961,1969,2),c("mean")),
    list("sec.industry",seq(1961,1969,2),c("mean")),
    list("sec.construction",seq(1961,1969,2),c("mean")),
    list("sec.services.venta",seq(1961,1969,2),c("mean")),
    list("sec.services.nonventa",seq(1961,1969,2),c("mean")),
    list("popdens",1969,c("mean")))
    ,treatment.identifier  = 17
    ,controls.identifier   = c(2:16,18)
    ,time.predictors.prior = c(1964:1969)
    ,time.optimize.ssr     = c(1960:1969)
    ,unit.names.variable   = c("regionname")
    ,time.plot            = c(1955:1997) 
    )

# 1. combine highest and second highest 
# schooling category and eliminate highest category
dataprep.out$X1["school.high",] <- 
 dataprep.out$X1["school.high",] + 
 dataprep.out$X1["school.post.high",]
dataprep.out$X1                 <- 
 as.matrix(dataprep.out$X1[
  -which(rownames(dataprep.out$X1)=="school.post.high"),])
dataprep.out$X0["school.high",] <- 
 dataprep.out$X0["school.high",] + 
 dataprep.out$X0["school.post.high",]
dataprep.out$X0                 <- 
dataprep.out$X0[
 -which(rownames(dataprep.out$X0)=="school.post.high"),]

# 2. make total and compute shares for the schooling catgeories
lowest  <- which(rownames(dataprep.out$X0)=="school.illit")
highest <- which(rownames(dataprep.out$X0)=="school.high")

dataprep.out$X1[lowest:highest,] <- 
 (100 * dataprep.out$X1[lowest:highest,]) /
 sum(dataprep.out$X1[lowest:highest,])
dataprep.out$X0[lowest:highest,] <-  
 100 * scale(dataprep.out$X0[lowest:highest,],
             center=FALSE,
             scale=colSums(dataprep.out$X0[lowest:highest,])
                                                 )
    
# run synth
synth.out <- synth(data.prep.obj = dataprep.out)

# Get result tables
synth.tables <- synth.tab(
                          dataprep.res = dataprep.out,
                          synth.res = synth.out
                          ) 

# results tables:
print(synth.tables)

# plot results:
# path
path.plot(synth.res = synth.out,
          dataprep.res = dataprep.out,
          Ylab = c("real per-capita GDP (1986 USD, thousand)"),
          Xlab = c("year"), 
          Ylim = c(0,13), 
          Legend = c("Basque country","synthetic Basque country"),
          ) 

## gaps
gaps.plot(synth.res = synth.out,
          dataprep.res = dataprep.out, 
          Ylab = c("gap in real per-capita GDP (1986 USD, thousand)"),
          Xlab = c("year"), 
          Ylim = c(-1.5,1.5), 
          )
          
## To create the placebo studies simply reassign
## the intervention to other units or times (see references for details)
}
}

\name{collect.optimx}
\alias{collect.optimx}
\title{
Collect results from optimx optimization methods
}
\description{
An internal function that collects the results from the different optimization methods run by optimx. It stores the parameter and function values and extracts the results for the best performing method (minimum or maximum).
}
\usage{
collect.optimx(res, opt = "min")
}
\arguments{
  \item{res}{
Output from a call to optimx().
}
  \item{opt}{
Either "min" or "max" to extract results for he methods that obtained the minimum or maximum function value 
across the methods.
}
}

\value{
  \item{out.list}{Dataframe with results from the different methods.}
  \item{par}{Parameter values from method that attained minimum/maximum across the methods.}
  \item{value}{Function value from method that attained minimum/maximum across the methods.}
}

\author{
Jens Hainmueller
}

\seealso{
Also see \code{\link[optimx]{optimx}}.
}

\name{spec.pred.func}
\alias{spec.pred.func}
\title{Special Predictor Function for Dataprep}
\description{ This function is called by \code{\link{dataprep}} to handle special predictors in the process of setting up the dataset to be loaded into \code{\link{synth}}. It should not be called manually by the normal user.
}
\usage{spec.pred.func(list.object = NULL,
          tr.numb = NULL,
          co.numb = NULL,
          unit.var = NULL,
          time.var = NULL,
          foo.object = NULL,
          X0.inner = NULL,
          X1.inner = NULL)
}
\arguments{
  \item{list.object}{NA}
  \item{tr.numb}{NA}
  \item{co.numb}{NA}
  \item{unit.var}{NA}
  \item{time.var}{NA}
  \item{foo.object}{NA}
  \item{X0.inner}{NA}
  \item{X1.inner}{NA}
}
\details{NA
}
\value{NA
}
\references{

Abadie, A., Diamond, A., Hainmueller, J. (2014). Comparative Politics and the Synthetic Control Method. \emph{American Journal of Political Science} Forthcoming 2014.

Synthetic : An R Package for Synthetic Control Methods in Comparative Case Studies. \emph{Journal of Statistical Software} 42 (13) 1--17.
       
Abadie, A., Diamond, A., Hainmueller, J. (2011). Synth: An R Package for Synthetic Control Methods in Comparative Case Studies. \emph{Journal of Statistical Software} 42 (13) 1--17.

Abadie A, Diamond A, Hainmueller J (2010). Synthetic Control Methods for Comparative Case Studies: Estimating the Effect of California's Tobacco Control Program. \emph{Journal of the American Statistical Association} 105 (490) 493--505.
     
Abadie, A. and Gardeazabal, J. (2003) Economic Costs of Conflict: A Case Study of the Basque Country \emph{American Economic Review} 93 (1) 113--132.
  
  }
   
\author{Jens Hainmueller and Alexis Diamond}
\seealso{ \code{\link{synth}}, \code{\link{dataprep}}, \code{\link{gaps.plot}}, \code{\link{path.plot}}, \code{\link{synth.tab}} }
